import pathlib
import json
import os


def loadData(fqfn: str, defaultData: dict | list = None) -> dict | list:
    """
    loads JSON data from the file specified

    Args:
        fqfn (str): file name, including path
        defaultData (_type_, optional): default data if the file does not exist. Defaults to None

    Returns:
        dict | list: JSON data from the file
    """
    if not os.path.exists(fqfn):
        return defaultData
    with open(fqfn) as rdr:
        return json.load(rdr)


def dumpData(fqfn: str, data: dict | list) -> str:
    """
    dumps the data to the file name specified

    Args:
        fqfn (str): file name, including path
        data (_type_): data to be saved to file as JSON

    Returns:
        str: file name, including path
    """
    pathlib.Path(os.path.split(fqfn)[0]).mkdir(parents=True, exist_ok=True)
    with open(fqfn, "w") as wtr:
        json.dump(data, wtr, indent=4)
    return fqfn



from kissut.testData.files import dumpData, loadData
import logging
import os

ALT_CFG_ENV_VAR = "KISSUT_CFG_FILE"


def _getPaths() -> dict:
    d = {"logCfgFile": "wkdir/unittest/logConfig.json", "testCfgFile": "wkdir/unittest/testConfig.json"}
    altPath = os.getenv(ALT_CFG_ENV_VAR)
    if altPath:
        altData = loadData(altPath, d)
        d.update(altData)
    return d


TEST_PATHS = _getPaths()


def logConfigFile() -> str:
    """
    returns the log configuration file path

    Returns:
        str: file path
    """
    return TEST_PATHS["logCfgFile"]


def loadConfigData(fqfn: str = TEST_PATHS["testCfgFile"], defaultData=None, logName: str = None) -> dict:
    """
    loads a unittest JSON config file from a standard location

    Args:
        fqfn (str, optional): file name of the configuration data
        defaultData (_type_, optional): the default data to save to disk if the file doesn't exist. Defaults to None
        logName (str, optional): the log name to use. Defaults to None

    Returns:
        dict: configuration data
    """
    if not os.path.exists(fqfn) and not defaultData:
        dumpData(fqfn, {"key": "value", "dbConn": "host=hostdb dbname=dev user=devusr password=doh!", "data": {}})
        logging.getLogger(logName).critical("Config File '{}' Didn't Exist, Edit Appropriately And Run Tests Again".format(fqfn))
        exit()
    return loadData(fqfn, defaultData)



from kissut.testData.text import randStr
from kissut.testData.files import dumpData, loadData
from kissut.testData.data import loadConfigData
from kissut.loggingTestCase.testCase import LoggingTestCase



from kissut.testData.files import loadData, dumpData
from kissut.loggingTestCase.logDirectory import initializeLogging
from kissut.testData.data import logConfigFile, loadConfigData
from kissut.testData.text import randStr
from types import FunctionType
from functools import wraps
import unittest
import logging

# ----
# loggingDecorator and MethodLoggingClass are used to wrap test methods and just add a logging line when entering


def loggingDecorator(testMethod):
    # logMethodAboutToRun is the wrapper function
    @wraps(testMethod)
    def logMethodAboutToRun(*args, **kwargs):
        lggr = logging.getLogger()
        # log the name of the test method about to be run
        lggr.debug("---- Running '{}' ----".format(testMethod.__name__))
        _result = testMethod(*args, **kwargs)
        # log that the test method has finished if desired
        return _result

    return logMethodAboutToRun


class MethodLoggingClass(type):
    def __new__(meta, classname, bases, classDict):
        newClassDict = {}
        for attributeName, attribute in classDict.items():
            if attributeName.startswith("test") and isinstance(attribute, FunctionType):
                # replace it with a wrapped version
                attribute = loggingDecorator(attribute)
            newClassDict[attributeName] = attribute
        return type.__new__(meta, classname, bases, newClassDict)


class LoggingTestCase(unittest.TestCase, metaclass=MethodLoggingClass):
    workingDirectory = None

    @classmethod
    def setUpClass(cls):
        if not LoggingTestCase.workingDirectory:
            LoggingTestCase.workingDirectory = initializeLogging(logConfigFilename=logConfigFile())
        cls.logger = logging.getLogger()
        cls.logger.debug("-- Entering {}".format(cls.__name__))

    @classmethod
    def tearDownClass(cls):
        # cls.logger.removeHandler(cls.handler)
        pass

    @classmethod
    def getConfig(cls) -> dict | list:
        """
        returns the configuration data from the working directory

        Returns:
            dict | list: configuration data from JSON
        """
        return loadConfigData()

    def loadWkDirData(self, filename: str = "data.json", defaultData: dict | list = None) -> dict | list:
        """
        prefixes the working directory to the filename provided, include subdirectories\n
        to the working directory if you like, ie: 'test_a/data.json

        Args:
            filename (str, optional): file name. Defaults to "data.json"
            defaultData (dict | list, optional): data to return if file does not exist. Defaults to None

        Returns:
            dict | list: data from JSON
        """
        fqfn = LoggingTestCase.workingDirectory + filename
        return loadData(fqfn, defaultData)

    def loadAnyData(self, fqfn: str, defaultData: dict | list = None) -> dict | list:
        """
        loads json data from anywhere, provide a fqfn

        Args:
            fqfn (str): file name and path
            defaultData (dict | list, optional): data to return if file does not exist. Defaults to None

        Returns:
            dict | list: data from JSON
        """
        return loadData(fqfn, defaultData)

    def dumpWkDirData(self, data: dict | list, filename="data.json") -> str:
        """
        prefixes the working directory to the filename provided, include subdirectories\n
        to the working directory if you like, ie: 'test_a/data.json

        Args:
            data (dict | list): data to save to JSON
            filename (str): file name

        Returns:
            str: file name and path
        """
        fqfn = LoggingTestCase.workingDirectory + filename
        return dumpData(fqfn, data)

    def dumpAnyData(self, data: dict | list, fqfn: str) -> str:
        """
        dumps data to JSON anywhere, provide a fully qualifed file name

        Args:
            data (dict | list): data to save to JSON
            fqfn (str): file name and path

        Returns:
            str: _description_
        """
        return dumpData(fqfn, data)

    def randString(self, size: int = 5):
        """
        returns a random string of length size

        Args:
            size (int, optional): length of the string returned. Defaults to 5

        Returns:
            str: random string
        """
        return randStr(size)



import os


def randStr(size=5) -> str:
    """
    returns a random string of length size

    Args:
        size (int, optional): length of the string returned. Defaults to 5

    Returns:
        str: random string
    """
    return str(os.urandom(size).hex())[:size]



LOG_DEFAULT_CONFIG = {
    "version": 1,
    "formatters": {"default": {"format": "%(levelname)-8s | %(asctime)s | %(module)-30s | %(funcName)-30s | %(message)s"}},
    "handlers": {
        "console": {"class": "logging.StreamHandler", "stream": "ext://sys.stdout", "formatter": "default"},
        "file": {
            "class": "logging.FileHandler",
            "filename": "wkdir/unittest/log.txt",
            "formatter": "default",
            "encoding": "utf-8",
        },
    },
    "root": {"level": "DEBUG", "handlers": ["file"]},
}



from kissut.loggingTestCase.defaultLogConfig import LOG_DEFAULT_CONFIG
from kissut.testData.files import loadData
from logging.config import dictConfig
import datetime
import pathlib
import os


def createUnittestWorkDir(directory: str) -> str:
    """creates the unittest working directory and subDirectory"""
    fullDir = "{}{}/".format(directory, datetime.datetime.now().strftime("%Y%m%dT%H%M%S"))
    pathlib.Path(fullDir).mkdir(parents=True, exist_ok=False)
    return fullDir


def _modifyRotatingFileName(filename: str, cfg: dict):
    if "handlers" in cfg:
        if "file" in cfg["handlers"]:
            if cfg["handlers"]["file"]["class"] == "logging.FileHandler":
                cfg["handlers"]["file"]["filename"] = filename


def initializeLogging(logConfigFilename: str) -> str:
    """returns the time-stamped unittest working directory"""
    wkDir = os.path.split(logConfigFilename)[0] + "/"
    wkDir = createUnittestWorkDir(wkDir)
    cfg = loadData(logConfigFilename, defaultData=dict(LOG_DEFAULT_CONFIG))
    _modifyRotatingFileName(wkDir + "log.txt", cfg)
    dictConfig(cfg)
    return wkDir



from setuptools import setup, find_packages
import json

# This file should not be edited when changing versions, edit the package metadata file
_metadata = json.load(open("kissut/_metadata.json", "r"))

setup(
    name=_metadata["name"],
    version=_metadata["version"],
    author=_metadata["author"],
    author_email=_metadata["author_email"],
    description=_metadata["description"],
    long_description=open("readme.md").read(),
    long_description_content_type="text/markdown",
    license=open("license.md").read(),
    package_data=_metadata["packageData"],
    include_package_data=True,
    packages=find_packages(),
    url=_metadata["url"],
    keywords=_metadata["keywords"],
    install_requires=open(_metadata["requirements"], "r").read().splitlines(),
    classifiers=_metadata["classifiers"],
)
